<?php
use \Firebase\JWT\JWT;
use Firebase\JWT\Key;

class ControllerAuth{
	private $config;
    private $authParams;
    private $ModelUsers;

	public function __construct() {
        $this->config = array(
            'clientId' => $_ENV["GOOGLE_CLIENT_ID"],
            'clientSecret' => $_ENV["GOOGLE_CLIENT_SECRET"],
            'authUrl' => 'https://accounts.google.com/o/oauth2/v2/auth',
            'tokenUrl' => 'https://oauth2.googleapis.com/token',
            'redirectUrl' => $_ENV["ORIGIN"] . '/auth/callback',
            'clientUrl' => $_ENV["ORIGIN"],
            'tokenSecret' => $_ENV["TOKEN"],
            'tokenExpiration' => 36000
        );

		$this->authParams = array(
            'client_id' => $this->config['clientId'],
            'redirect_uri' => $this->config['redirectUrl'],
            'response_type' => 'code',
            'scope' => 'openid profile email',
            'access_type' => 'offline',  
            'state' => 'standard_oauth',
            'prompt' => 'consent',
        );

        $this->ModelUsers = new ModelUsers('users');
    }

    function url($f3){
        try {
            $queryString = http_build_query($this->authParams);
            $authUrl = $this->config['authUrl'];
            $data = array(
                'url' => "$authUrl?$queryString"
            );
        
            echo json_encode($data);
        } catch (Exception $exception){
            $f3->error(500, "Erro interno: " . $exception->getMessage());
        }		
    }

    function token($f3){
        try {
            $code = $_GET['code'];
            
            if(!$code) $f3->error(500, "Parametro code não informado!");

            $tokenParam = http_build_query(array(
                'client_id' => $this->config['clientId'],
                'client_secret' => $this->config['clientSecret'],
                'code' => $code,
                'grant_type' => 'authorization_code',
                'redirect_uri' => $this->config['redirectUrl'],
            ));

            // CURL
            $options = array(
                CURLOPT_URL => $this->config['tokenUrl'],
                CURLOPT_POST => 1,
                CURLOPT_POSTFIELDS => $tokenParam,
                CURLOPT_RETURNTRANSFER => true,
            );

            // Inicializar a sessão cURL
            $ch = curl_init();
            curl_setopt_array($ch, $options);

            // Executar a solicitação cURL
            $response = curl_exec($ch);

            // Verificar se houve algum erro
            if (curl_errno($ch)) {
                $f3->error(500, "Erro na solicitação para api google");
            }

            $responseData = json_decode($response, true);

            list($header, $payload, $signature) = explode('.', $responseData['id_token']);
            $data = json_decode(base64_decode($payload),true);

            $result = $this->ModelUsers->find($columns=array(), $filters=array('e_mail' => $data['email']));
            
            $user = null;

            if($result){
                $user = array(
                    'id' => $result[0]['id'],
                    'e_mail' => $result[0]['e_mail'],
                    'name' => $result[0]['name'],
                    'admin' => $result[0]['admin'],
                    'exp' => time() + $this->config['tokenExpiration']
                );

            }else {
                $user_id = $this->ModelUsers->insert($columns=array('e_mail', 'name', 'admin'),
										  $values=array(array($data['email'], $data['name'], 0))
										);
                $user = array(
                    'id' => $user_id,
                    'e_mail' => $result[0]['e_mail'],
                    'name' => $result[0]['name'],
                    'admin' => $result[0]['admin'],
                    'exp' => time() + $this->config['tokenExpiration']
                );
            }



            $token = JWT::encode($user, $this->config['tokenSecret'], 'HS256');

            setcookie('token', $token, $user['exp'], '/', '', false, true);

            echo json_encode($user);
        } catch (Exception $exception){
            $f3->error(500, "Erro interno: " . $exception->getMessage());
        }
    }

    function logged_in($f3){
        try {
            if (isset($_COOKIE['token'])) {
                $token = $_COOKIE['token'];
                
                $user = JWT::decode($token, new Key($this->config['tokenSecret'], 'HS256'));
                $user->exp = time() + $this->config['tokenExpiration'];
                
                $token = JWT::encode((array) $user,  $this->config['tokenSecret'], 'HS256');
                setcookie('token', $token, $user->exp, '/', '', false, true);
                
                echo json_encode(array('loggedIn' => true, 'admin' => boolval($user->admin)));

            } else{
                echo json_encode(array('loggedIn' => false, 'admin' => false));
            }
        }
        catch (Exception $exception){
            setcookie('token', '', 0 ,'/', '', false, true);
            $f3->error(500, "Erro interno: " . $exception->getMessage());
            
        }
    }
    
    function logout($f3){
        try {
            setcookie('token', '', 0 ,'/', '', false, true);
        }
        catch (Exception $exception){
            $f3->error(500, "Erro interno: " . $exception->getMessage());
        }
    }

    function generateToken($f3){
        $user_id = $_GET['user_id'];
        
        $ModelUsers = new ModelUsers('users');
        $result = $ModelUsers->find($columns=array(), $filters=array('id' => $user_id));
    
        if($result){
            $user = array(
                'id' => $result[0]['id'],
                'e_mail' => $result[0]['e_mail'],
                'name' => $result[0]['name'],
                'admin' => $result[0]['admin'],
                'exp' => time() + 33600
            );
    
            $token = JWT::encode($user, $this->config['tokenSecret'], 'HS256');
    
            setcookie('token', $token, $user['exp'], '/', '', false, true);
        
            echo 'foi';
        } else {
            echo 'Não foi';
        }
    }
}
?>