<?php

class ControllerUserJobs extends User{
	public function __construct() {
        parent::__construct();
		$this->SlurmAPI = new SlurmAPI();
    }

    public function post($f3) {
		try {
			$user = $f3->get('user');

			$data = json_decode(file_get_contents('php://input'), true);	

			$queue = $data['queue'];
			$project_slug = $data['project'];
			$image = $data['image'];
			$name = $data['name'];
			$commands = $data['commands'];
			$type = $data['type'] == 'paralelo' ? 0 : 1;


			$cluster = $this->ModelClusters->findClusterByQueueAndUserID($user['id'], $queue);
			if(!$cluster) $f3->error(403, "Submissão não autorizada.");

			$project = $this->ModelProjects->find(
				$columns=array('id', 'nfs_ip', 'nfs_path'), 
				$filters=array('slug' => $project_slug)
			)[0];

			$job_id = $this->ModelJobs->insert(
				$columns=array(
					'user_id', 
					'project_id', 
					'name', 
					'type',
					'queue', 
					'image', 
				),
				$values=array(
					array(
						$user['id'], 
						$project['id'], 
						$name, 
						$type,
						$queue, 
						$image, 
					)
				)
			);

			if(intval($job_id) == 0) $f3->error(500, "Ocorreu um erro durante o cadastro do trabalho!");

			$cmds = array();
			$lenght = count($commands);
			for ($i = 0; $i < $lenght; $i++) {
				array_push($cmds, array($job_id, $i, $commands[$i], -1));
			}

			$this->ModelCommands->insert($columns=array('job_id', 'id_slurm', 'command', 'state'), $values=$cmds);

			echo $job_id;
				

		} catch (Exception $exception){
            $f3->error(500, "Erro interno: " . $exception->getMessage());
        }
    }

	public function create($f3) {
		try {
			$user = $f3->get('user');

			$results = $this->ModelProjects->findByUserId($user['id'], array('id','slug'));

			$projects = array();
			for ($i = 0; $i < count($results); $i++) {
				$projects[$results[$i]['slug']]['queues'] = $this->ModelQueues->findByProjectId($results[$i]['id']);
				$projects[$results[$i]['slug']]['images'] = $this->ModelImages->findByProjectId($results[$i]['id']);

			}

			echo json_encode($projects);

		} catch (Exception $exception){
            $f3->error(500, "Erro interno: " . $exception->getMessage());
        }		

	}

	public function list($f3) {
		try {
			$user = $f3->get('user');
			

			$jobs = $this->ModelJobs->findByUserId($user['id']);
			
			echo json_encode($jobs);
		} catch (Exception $exception){
            $f3->error(500, "Erro interno: " . $exception->getMessage());
        }	

	}

	public function view($f3) {
		try {
			$user = $f3->get('user');
			$job_id = $_GET['job_id'];


			$job = $this->ModelJobs->findByUserIdAndJobId($user['id'], $job_id);

			if(!$job) $f3->error(403, "Acesso aos dados negado!");


			$commands = $this->ModelCommands->find($columns=array('state', 'command'),
													$filters=array('job_id' => $job_id));

			$job['commands'] =  $commands;

			echo json_encode($job);

	
		} catch (Exception $exception){
            $f3->error(500, "Erro interno: " . $exception->getMessage());
        }
	}

	public function cancel($f3) {
		try {
			$user = $f3->get('user');
			$data = json_decode(file_get_contents('php://input'), true);
			$job_id = $data['job_id'];
	
			$job = $this->ModelJobs->find($columns=array('id'),
											  $filters=array('id' => $job_id, 'user_id' => $user['id']));

			if(!$job) $f3->error(403, "Cancelamento não autorizado!");
			

			$api = new API($_ENV["QUEUE_API"], 'token');
			$api->delete("/jobs/{$job_id}");
		

		} catch (Exception $exception){
			$f3->error(500, "Erro interno: " . $exception->getMessage());
		}
		
	}

}
?>