<?php

class ModelJobs extends ModelBase{
    
    public function findAllByState() {
        $sql  = '
            SELECT jobs.id as job_id, commands.id_slurm, clusters.name as cluster_name, clusters.url as cluster_url, clusters.token as cluster_token
            FROM jobs
            LEFT JOIN commands ON commands.job_id = jobs.id
            LEFT JOIN queues ON queues.name = jobs.queue
            LEFT JOIN clusters ON clusters.id = queues.cluster_id
            WHERE commands.state in (0,1,7) AND commands.id_slurm != -1;';


        #Fas busca no DB.
        $result = $this->DB->select(strval($sql));

        #Retorna resultados.
        $list = array();
        while ($row = mysqli_fetch_assoc($result)) {
            array_push($list, $row);
        }

        return $list;

    }

    public function findAll(){
        $sql = "SELECT  jobs.id as id,
                        jobs.name as name, 
                        users.e_mail as user,
                        jobs.queue as queue, 
                        CASE jobs.type
                            WHEN 0 THEN 'Paralelo'
                            WHEN 1 THEN 'Distribuido'
                        END AS type,
                        projects.slug as project, 
                        jobs.image as image,
                        commands.n_commands as n_commands,
                        CASE
                            WHEN commands.state = 1 THEN 'Executando'
                            WHEN commands.state = 2 THEN 'Interrompido'
                            WHEN commands.state = 3 THEN 'Falha'
                            WHEN commands.state = 4 THEN 'Cancelado'
                            WHEN commands.state = 5 THEN 'Completo'
                            ELSE 'Aguardando'
                        END AS state
                FROM jobs
                LEFT JOIN projects ON jobs.project_id = projects.id
                LEFT JOIN (SELECT job_id,
                                COUNT(id_slurm) as n_commands,
                                MIN(CASE
                                        WHEN state = 1 THEN 1
                                        WHEN state = 7 then 2
                                        WHEN state = 5 THEN 3
                                        WHEN state = 4 THEN 4
                                        WHEN state = 3 THEN 5
                                        ELSE 6
                                END) AS state
                                FROM commands
                                GROUP BY job_id) as commands ON jobs.id = commands.job_id
                LEFT JOIN users on users.id = jobs.user_id
                ORDER BY jobs.id DESC;";
        

        #Fas busca no DB.
        $result = $this->DB->select(strval($sql));

        #Retorna resultados.
        $list = array();
        while ($row = mysqli_fetch_assoc($result)) {
            array_push($list, $row);
        }

        return $list;
        
    }

    public function findByUserId($user_id){
        $sql = "SELECT  jobs.id as id,
                        jobs.name as name, 
                        jobs.queue as queue, 
                        CASE jobs.type
                            WHEN 0 THEN 'Paralelo'
                            WHEN 1 THEN 'Distribuido'
                        END AS type,
                        projects.slug as project, 
                        jobs.image as image,
                        commands.n_commands as n_commands,
                        CASE
                            WHEN commands.state = 1 THEN 'Executando'
                            WHEN commands.state = 2 THEN 'Interrompido'
                            WHEN commands.state = 3 THEN 'Falha'
                            WHEN commands.state = 4 THEN 'Cancelado'
                            WHEN commands.state = 5 THEN 'Completo'
                            ELSE 'Aguardando'
                        END AS state
                FROM jobs
                LEFT JOIN projects ON jobs.project_id = projects.id
                LEFT JOIN (SELECT job_id,
                                COUNT(id_slurm) as n_commands,
                                MIN(CASE
                                        WHEN state = 1 THEN 1
                                        WHEN state = 7 then 2
                                        WHEN state = 5 THEN 3
                                        WHEN state = 4 THEN 4
                                        WHEN state = 3 THEN 5
                                        ELSE 6
                                END) AS state
                                FROM commands
                                GROUP BY job_id) as commands ON jobs.id = commands.job_id
                WHERE jobs.user_id = $user_id
                ORDER BY jobs.id DESC;";

        #Faz busca no DB.
        $result = $this->DB->select(strval($sql));

        #Retorna resultados.
        $list = array();
        while ($row = mysqli_fetch_assoc($result)) {
            array_push($list, $row);
        }

        return $list;
        
    }

    public function findByUserIdAndJobId($user_id, $job_id){
        $sql = "SELECT  jobs.id as id,
                        jobs.user_id,
                        jobs.name as name, 
                        jobs.queue as queue, 
                        CASE jobs.type
                            WHEN 0 THEN 'Paralelo'
                            WHEN 1 THEN 'Distribuido'
                        END AS type,
                        projects.slug as project, 
                        jobs.image as image,
                        FROM_UNIXTIME(commands.time_submit) as time_submit,
                        FROM_UNIXTIME(commands.time_start) as time_start,
                        FROM_UNIXTIME(commands.time_end) as time_end,
                        CASE
                            WHEN commands.state = 1 THEN 'Executando'
                            WHEN commands.state = 2 THEN 'Interrompido'
                            WHEN commands.state = 3 THEN 'Falha'
                            WHEN commands.state = 4 THEN 'Cancelado'
                            WHEN commands.state = 5 THEN 'Completo'
                            ELSE 'Aguardando'
                        END AS state
                FROM jobs
                LEFT JOIN projects ON jobs.project_id = projects.id
                LEFT JOIN (SELECT job_id,
                    MIN(time_submit) AS time_submit,
                    MIN(CASE
                        WHEN time_start > 0 THEN time_start
                        ELSE NULL
                    END) AS time_start,
                    MAX(time_end) AS time_end,
                    MIN(CASE
                        WHEN state = 1 THEN 1
                        WHEN state = 7 then 2
                        WHEN state = 5 THEN 3
                        WHEN state = 4 THEN 4
                        WHEN state = 3 THEN 5
                        ELSE 6
                    END) AS state
                FROM commands
                GROUP BY job_id) as commands ON jobs.id = commands.job_id
                WHERE jobs.id = $job_id and jobs.user_id = $user_id;";

        #Faz busca no DB.
        $result = $this->DB->select(strval($sql));
        $job = $result->fetch_assoc();

        #Calcula tempo executando.
        if ($job['time_start']){
            $time_start = new DateTime($job['time_start']);
            if($job['state'] == 'Executando'){
                $time_end =  new DateTime();
            } else {
                $time_end =  new DateTime($job['time_end']);
            }
                
                
            $time_running = $time_start->diff($time_end);

            $job['time_running'] = $time_running->format('%d dias, %h horas, %i minutos, %s segundos');
        }
        return $job;

    }

    public function findById($job_id){
        $sql = "SELECT  jobs.id as id,
                        jobs.user_id,
                        jobs.name as name, 
                        jobs.queue as queue, 
                        CASE jobs.type
                            WHEN 0 THEN 'Paralelo'
                            WHEN 1 THEN 'Distribuido'
                        END AS type,
                        projects.slug as project, 
                        jobs.image as image,
                        FROM_UNIXTIME(commands.time_submit) as time_submit,
                        FROM_UNIXTIME(commands.time_start) as time_start,
                        FROM_UNIXTIME(commands.time_end) as time_end,
                        CASE
                            WHEN commands.state = 1 THEN 'Executando'
                            WHEN commands.state = 2 THEN 'Interrompido'
                            WHEN commands.state = 3 THEN 'Falha'
                            WHEN commands.state = 4 THEN 'Cancelado'
                            WHEN commands.state = 5 THEN 'Completo'
                            ELSE 'Aguardando'
                        END AS state
                FROM jobs
                LEFT JOIN projects ON jobs.project_id = projects.id
                LEFT JOIN (SELECT job_id,
                    MIN(time_submit) AS time_submit,
                    MIN(CASE
                        WHEN time_start > 0 THEN time_start
                        ELSE NULL
                    END) AS time_start,
                    MAX(time_end) AS time_end,
                    MIN(CASE
                        WHEN state = 1 THEN 1
                        WHEN state = 7 then 2
                        WHEN state = 5 THEN 3
                        WHEN state = 4 THEN 4
                        WHEN state = 3 THEN 5
                        ELSE 6
                    END) AS state
                FROM commands
                GROUP BY job_id) as commands ON jobs.id = commands.job_id
                WHERE jobs.id = $job_id;";

        #Faz busca no DB.
        $result = $this->DB->select(strval($sql));
        $job = $result->fetch_assoc();

        #Calcula tempo executando.
        if ($job['time_start']){
            $time_start = new DateTime($job['time_start']);
            if($job['state'] == 'Executando'){
                $time_end =  new DateTime();
            } else {
                $time_end =  new DateTime($job['time_end']);
            }
                
                
            $time_running = $time_start->diff($time_end);

            $job['time_running'] = $time_running->format('%d dias, %h horas, %i minutos, %s segundos');
        }
        return $job;

    }

    public function findByQueue($queue){
        $sql1 = 'SET @position:=0;';
        
        $sql = "SELECT @position AS position, jobs.id, projects.slug as project, jobs.name, jobs.state, jobs.n_commands, jobs.time_submit, jobs.time_start
                FROM (
                    SELECT  jobs.id, 
                            jobs.project_id,
                            jobs.name,
                            CASE
                                WHEN MAX(commands.state) = 1 THEN 'Executando'
                                ELSE 'Aguardando'
                        	END AS state,
                            COUNT(commands.id_slurm) as n_commands, 
                            FROM_UNIXTIME(MIN(commands.time_submit)) as time_submit, 
                            FROM_UNIXTIME(MAX(commands.time_start)) as time_start
                        FROM jobs
                        LEFT JOIN commands on jobs.id = commands.job_id
                        WHERE (commands.state = 1 or commands.state = 0) and jobs.queue = '$queue'
                        GROUP BY jobs.name, jobs.id	
                        ORDER BY time_submit ASC
                ) as jobs
                LEFT JOIN projects on jobs.project_id = projects.id
                WHERE jobs.state = 'Executando'
                UNION
                SELECT @position:=@position+1 AS position, jobs.id, projects.slug as project, jobs.name, jobs.state, jobs.n_commands, jobs.time_submit, jobs.time_start
                FROM (
                    SELECT  jobs.id, 
                            jobs.project_id,
                            jobs.name,
                            CASE
                                WHEN MAX(commands.state) = 1 THEN 'Executando'
                                ELSE 'Aguardando'
                        	END AS state,
                            COUNT(commands.id_slurm) as n_commands, 
                            FROM_UNIXTIME(MIN(commands.time_submit)) as time_submit, 
                            FROM_UNIXTIME(MAX(commands.time_start)) as time_start
                        FROM jobs
                        LEFT JOIN commands on jobs.id = commands.job_id
                        WHERE (commands.state = 1 or commands.state = 0) and jobs.queue = '$queue'
                        GROUP BY jobs.name, jobs.id	
                        ORDER BY time_submit ASC
                ) as jobs
                LEFT JOIN projects on jobs.project_id = projects.id
                WHERE jobs.state = 'Aguardando';";

        
        #Faz busca no DB.
        $result = $this->DB->updateN(array(strval($sql1), strval($sql)));

        #Retorna resultados.
        $list = array();
        while ($row = mysqli_fetch_assoc($result)) {
            if($row['time_start']){
                $time_start = new DateTime($row['time_start']);
                $time_end = new DateTime();
                $time_running = $time_start->diff($time_end);
                $row['time_running'] = $time_running->format('%d dias, %h horas, %i minutos, %s segundos');
            } else {
                $row['time_running'] = '-';
            }

            array_push($list, $row);
        }

        return $list;

    }
}
?>