<?php
require 'vendor/autoload.php';
require 'config/db.php';

require 'app/models/base.php';
require 'app/models/jobs.php';
require 'app/models/commands.php';
require 'app/models/clusters.php';
require 'app/models/projects.php';
require 'app/models/users.php';
require 'app/models/queues.php';
require 'app/models/images.php';

require 'app/APIs/slurm.php';
require 'app/utils/utilities.php';

require 'app/controllers/auth.php';
require 'app/controllers/user/user.php';
require 'app/controllers/user/jobs.php';
require 'app/controllers/user/queues.php';
require 'app/controllers/admin/admin.php';
require 'app/controllers/admin/jobs.php';
require 'app/controllers/admin/queues.php';
require 'app/controllers/admin/projects.php';
require 'app/controllers/admin/images.php';
require 'app/controllers/admin/clusters.php';
require 'app/controllers/admin/admins.php';

#Server
$f3 = \Base::instance();
$f3->set('CACHE', TRUE);
$f3->set('ONERROR',
    function($f3) {
        echo $f3->get('ERROR.text');
    }
);

header('Access-Control-Allow-Origin: ' . $_ENV["ORIGIN"]);
header("Access-Control-Allow-Methods: GET, POST, PUT, DELETE");
header("Access-Control-Allow-Headers: Content-Type, Authorization");
header("Access-Control-Allow-Credentials: true");


if ($_SERVER['REQUEST_METHOD'] === 'OPTIONS') {
    http_response_code(200);
    exit();
}


$token = $_SERVER['HTTP_AUTHORIZATION'] ?? $f3->error( 400, "Token de autenticacao não encontrado!" );
if ( $_ENV["TOKEN"] != $token ) $f3->error( 401, "Token Invalido!" );

$prefix = $_ENV["PREFIX"];

#Rotas
// ** AUTENTICAÇÃO ** \\
$f3->route("GET $prefix/auth/url","ControllerAuth->url");
$f3->route("GET $prefix/auth/token","ControllerAuth->token");
$f3->route("GET $prefix/auth/logged_in","ControllerAuth->logged_in");
$f3->route("POST $prefix/auth/logout","ControllerAuth->logout");
$f3->route("GET $prefix/auth/teste","ControllerAuth->generateToken");


// ** SOMENTE USUARIO ** \\
//Filas
$f3->route("GET $prefix/usuario/filas", "ControllerUserQueues->list");
$f3->route("GET $prefix/usuario/filas/visualizar", "ControllerUserQueues->view");

//Trabalhos
$f3->route("GET $prefix/usuario/trabalhos/novo", "ControllerUserJobs->create");
$f3->route("POST $prefix/usuario/trabalhos/novo","ControllerUserJobs->post");
$f3->route("GET $prefix/usuario/trabalhos", "ControllerUserJobs->list");
$f3->route("GET $prefix/usuario/trabalhos/visualizar", "ControllerUserJobs->view");
$f3->route("PUT $prefix/usuario/trabalhos/cancelar", "ControllerUserJobs->cancel");

// ** SOMENTE Admin ** \\
//Filas
$f3->route("GET $prefix/admin/filas", "ControllerAdminQueues->list");

$f3->route("POST $prefix/admin/filas/novo","ControllerAdminQueues->post");
$f3->route("PUT $prefix/admin/filas/editar","ControllerAdminQueues->put");
$f3->route("DELETE $prefix/admin/filas/deletar","ControllerAdminQueues->delete");
$f3->route("GET $prefix/admin/filas/visualizar", "ControllerAdminQueues->view");

//Trabalhos
$f3->route("GET $prefix/admin/trabalhos/novo", "ControllerAdminJobs->create");
$f3->route("POST $prefix/admin/trabalhos/novo","ControllerAdminJobs->post");
$f3->route("GET $prefix/admin/trabalhos", "ControllerAdminJobs->list");
$f3->route("GET $prefix/admin/trabalhos/visualizar", "ControllerAdminJobs->view");
$f3->route("PUT $prefix/admin/trabalhos/cancelar", "ControllerAdminJobs->cancel");

//Projetos
$f3->route("GET $prefix/admin/projetos", "ControllerAdminProjects->list");
$f3->route("POST $prefix/admin/projetos/novo","ControllerAdminProjects->post");
$f3->route("PUT $prefix/admin/projetos/editar","ControllerAdminProjects->put");
$f3->route("DELETE $prefix/admin/projetos/deletar","ControllerAdminProjects->delete");
$f3->route("GET $prefix/admin/projetos/visualizar", "ControllerAdminProjects->view");
$f3->route("POST $prefix/admin/projetos/participante","ControllerAdminProjects->insertParticipant");
$f3->route("DELETE $prefix/admin/projetos/participante","ControllerAdminProjects->deleteParticipant");

//Imagens
$f3->route("GET $prefix/admin/imagens", "ControllerAdminImages->list");
$f3->route("POST $prefix/admin/imagens/novo","ControllerAdminImages->post");
$f3->route("PUT $prefix/admin/imagens/editar","ControllerAdminImages->put");
$f3->route("DELETE $prefix/admin/imagens/deletar","ControllerAdminImages->delete");
$f3->route("GET $prefix/admin/imagens/visualizar", "ControllerAdminImages->view");
$f3->route("POST $prefix/admin/imagens/projeto","ControllerAdminImages->insertProject");
$f3->route("DELETE $prefix/admin/imagens/projeto","ControllerAdminImages->deleteProject");

//Clusters
$f3->route("GET $prefix/admin/clusters", "ControllerAdminClusters->list");
$f3->route("POST $prefix/admin/clusters/novo","ControllerAdminClusters->post");
$f3->route("PUT $prefix/admin/clusters/editar","ControllerAdminClusters->put");
$f3->route("DELETE $prefix/admin/clusters/deletar","ControllerAdminClusters->delete");
$f3->route("GET $prefix/admin/clusters/visualizar", "ControllerAdminClusters->view");
$f3->route("POST $prefix/admin/clusters/projeto","ControllerAdminClusters->insertProject");
$f3->route("DELETE $prefix/admin/clusters/projeto","ControllerAdminClusters->deleteProject");

//Admins
$f3->route("GET $prefix/admin/admins", "ControllerAdminAdmins->list");
$f3->route("PUT $prefix/admin/admins", "ControllerAdminAdmins->put");
$f3->route("DELETE $prefix/admin/admins", "ControllerAdminAdmins->delete");

$f3->run();